
from __future__ import absolute_import

from . import  (test_dataset_getitem,
                test_dataset_swmr,
                test_dims_dimensionproxy,
                test_file,
                test_filters,
                test_attribute_create,
                test_threads,
                test_datatype,
                test_vds)

MODULES = ( test_dataset_getitem,
            test_dataset_swmr,
            test_dims_dimensionproxy,
            test_file,
            test_filters,
            test_attribute_create,
            test_threads,
            test_datatype,
            test_vds)
